<?php

namespace SovereignStack\SecuritySafe;

// Prevent Direct Access
( defined( 'ABSPATH' ) ) || die;

/**
 * @package SecuritySafe
 * @since   2.4.3
 */
class REQUEST {

	/**
	 * Retrieves and sanitizes the supplied key as a boolean.
	 *
	 * @param string $key
	 *
	 * @return bool
	 */
	public static function bool( $key = '' ) {

		return isset( $_REQUEST[ $key ] ) && $_REQUEST[ $key ];

	}

	/**
	 * Retrieves and sanitizes the supplied key as a text field.
	 *
	 * @return string
	 */
	public static function text_field( $key = '') {

		return isset( $_REQUEST[ $key ] ) ? sanitize_text_field( $_REQUEST[ $key ] ) : '';

	}

	/**
	 * Retrieves and sanitizes the supplied key as an integer.
	 *
	 * @param string $key
	 *
	 * @return int
	 */
	public static function int( $key = '', $default = 0 ) {

		return isset( $_REQUEST[ $key ] ) ? (int) $_REQUEST[ $key ] : $default;

	}

	/**
	 * Retrieves and sanitizes the supplied key as a key string.
	 *
	 * @param string $key
	 *
	 * @return string
	 */
	public static function key( $key = '', $default = '', $prepend = '', $append = '' ) {

		return isset( $_REQUEST[ $key ] ) ? trim( $prepend . sanitize_key( $_REQUEST[ $key ] ) . $append ) : $default;

	}

	/**
	 * Retrieves and sanitizes the supplied key's array values
	 *
	 * @param string $key
	 * @param array  $default
	 * @param string $sanitize
	 *
	 * @return array
	 */
	public static function arr( $key = '', $default = [], $sanitize = 'text_field' ) {

		$array = [];

		if ( isset( $_REQUEST[ $key ] ) && is_array( $_REQUEST[ $key ] ) ) {

			// Sanitize the array values
			if ( ! empty( $_REQUEST[ $key ] ) ) {

				if ( 'text_field' === $sanitize ) {
					$array = map_deep( $_REQUEST[ $key ], 'sanitize_text_field' );
				} elseif ( 'key' === $sanitize ) {
					$array = map_deep( $_REQUEST[ $key ], 'sanitize_key' );
				} elseif ( 'int' === $sanitize ) {
					$array[] = map_deep( $_REQUEST[ $key ], 'intval' );
				} elseif ( 'bool' === $sanitize ) {
					$array[] = map_deep( $_REQUEST[ $key ], 'boolval' );
				}

			}

		} elseif ( isset( $_REQUEST[ $key ] ) && ! is_array( $_REQUEST[ $key ] ) && $_REQUEST[ $key ] ) {

			// Sanitize and create an array
			if ( 'text_field' === $sanitize ) {
				$array[] = sanitize_text_field( $_REQUEST[ $key ] );
			} elseif ( 'key' === $sanitize ) {
				$array[] = sanitize_key( $_REQUEST[ $key ] );
			} elseif ( 'int' === $sanitize ) {
				$array[] = (int) $_REQUEST[ $key ];
			} elseif ( 'bool' === $sanitize ) {
				$array[] = (bool) $_REQUEST[ $key ];
			}

		} else {
			// Fallback on the default value
			$array = $default;

			// Set the original with the default value to make compatible with saving empty settings values
			$_REQUEST[ $key ] = $default;
		}

		return $array;

	}

	/**
	 * Retrieves and sanitizes the entire $_POST array
	 *
	 * @return array
	 */
	public static function POST() {

		$array = [];

		if ( isset( $_POST ) && ! empty( $_POST ) ) {
			$array = map_deep( $_POST, 'sanitize_text_field' );
		}

		return $array;

	}

}